# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest Time Entry views
"""

from rattail_harvest.db.model import HarvestTimeEntry

from .master import HarvestMasterView


class HarvestTimeEntryView(HarvestMasterView):
    """
    Master view for Harvest Time Entries
    """
    model_class = HarvestTimeEntry
    url_prefix = '/harvest/time-entries'
    route_prefix = 'harvest.time_entries'

    labels = {
        'user_id': "User ID",
        'client_id': "Client ID",
        'project_id': "Project ID",
        'task_id': "Task ID",
        'invoice_id': "Invoice ID",
    }

    grid_columns = [
        'id',
        'spent_date',
        'user',
        'client',
        'project',
        'task',
        'hours',
        'notes',
    ]

    def configure_grid(self, g):
        super(HarvestTimeEntryView, self).configure_grid(g)

        g.set_type('hours', 'duration_hours')

        g.set_sort_defaults('spent_date', 'desc')

        g.set_link('id')
        g.set_link('user')
        g.set_link('client')
        g.set_link('notes')


def includeme(config):
    HarvestTimeEntryView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest User views
"""

from rattail_harvest.db.model import HarvestUser

from .master import HarvestMasterView


class HarvestUserView(HarvestMasterView):
    """
    Master view for Harvest Users
    """
    model_class = HarvestUser
    url_prefix = '/harvest/users'
    route_prefix = 'harvest.users'

    grid_columns = [
        'id',
        'first_name',
        'last_name',
        'email',
        'telephone',
        'timezone',
        'is_admin',
    ]

    def configure_grid(self, g):
        super(HarvestUserView, self).configure_grid(g)

        g.set_sort_defaults('first_name')

        g.set_link('id')
        g.set_link('first_name')
        g.set_link('last_name')
        g.set_link('email')

    def configure_form(self, f):
        super(HarvestUserView, self).configure_form(f)

        # TODO: should add this as child rows/grid instead
        f.remove('time_entries')


def includeme(config):
    HarvestUserView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest Project views
"""

from rattail_harvest.db.model import HarvestProject

from .master import HarvestMasterView


class HarvestProjectView(HarvestMasterView):
    """
    Master view for Harvest Projects
    """
    model_class = HarvestProject
    url_prefix = '/harvest/projects'
    route_prefix = 'harvest.projects'

    grid_columns = [
        'id',
        'client',
        'name',
        'code',
        'is_active',
        'is_billable',
        'bill_by',
        'hourly_rate',
        'fee',
    ]

    def configure_grid(self, g):
        super(HarvestProjectView, self).configure_grid(g)
        model = self.model

        g.set_joiner('client', lambda q: q.outerjoin(model.HarvestClient))
        g.set_sorter('client', model.HarvestClient.name)
        g.set_filter('client', model.HarvestClient.name, label="Client Name")
        g.filters['client'].default_active = True
        g.filters['client'].default_verb = 'contains'

        g.set_type('hourly_rate', 'currency')
        g.set_type('fee', 'currency')

        g.set_sort_defaults('client')

        g.set_link('id')
        g.set_link('client')
        g.set_link('name')
        g.set_link('code')


def includeme(config):
    HarvestProjectView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
tailbone-harvest setup script
"""

import os
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))
exec(open(os.path.join(here, 'tailbone_harvest', '_version.py')).read())
README = open(os.path.join(here, 'README.rst')).read()


requires = [
    #
    # Version numbers within comments below have specific meanings.
    # Basically the 'low' value is a "soft low," and 'high' a "soft high."
    # In other words:
    #
    # If either a 'low' or 'high' value exists, the primary point to be
    # made about the value is that it represents the most current (stable)
    # version available for the package (assuming typical public access
    # methods) whenever this project was started and/or documented.
    # Therefore:
    #
    # If a 'low' version is present, you should know that attempts to use
    # versions of the package significantly older than the 'low' version
    # may not yield happy results.  (A "hard" high limit may or may not be
    # indicated by a true version requirement.)
    #
    # Similarly, if a 'high' version is present, and especially if this
    # project has laid dormant for a while, you may need to refactor a bit
    # when attempting to support a more recent version of the package.  (A
    # "hard" low limit should be indicated by a true version requirement
    # when a 'high' version is present.)
    #
    # In any case, developers and other users are encouraged to play
    # outside the lines with regard to these soft limits.  If bugs are
    # encountered then they should be filed as such.
    #
    # package                           # low                   high

    'invoke',                           # 1.5.0
    'rattail-harvest',                  # 0.1.0
    'Tailbone',                         # 0.8.199
]


setup(
    name = "tailbone-harvest",
    version = __version__,
    author = "Lance Edgar",
    author_email = "lance@edbob.org",
    url = "https://rattailproject.org/",
    description = "Tailbone integration package for Harvest",
    long_description = README,

    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Office/Business',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    install_requires = requires,
    packages = find_packages(),
    include_package_data = True,
)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest Task views
"""

from rattail_harvest.db.model import HarvestTask

from .master import HarvestMasterView


class HarvestTaskView(HarvestMasterView):
    """
    Master view for Harvest Tasks
    """
    model_class = HarvestTask
    url_prefix = '/harvest/tasks'
    route_prefix = 'harvest.tasks'

    grid_columns = [
        'id',
        'name',
        'billable_by_default',
        'default_hourly_rate',
        'is_default',
        'is_active',
    ]

    def configure_grid(self, g):
        super(HarvestTaskView, self).configure_grid(g)

        g.set_sort_defaults('name')

        g.set_link('id')
        g.set_link('name')

    def configure_form(self, f):
        super(HarvestTaskView, self).configure_form(f)

        # time_entries
        f.remove_field('time_entries')


def includeme(config):
    HarvestTaskView.defaults(config)
